/*
 * $RCSfile: ImagePanel.java,v $
 *
 * Copyright (c) 1999-2002. Jens Bohl. All rights reserved.
 *
 * This software is published under the GPL GNU General Public License.
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * http://www.resmedicinae.org
 * - Information in Medicine -
 */

package org.resmedicinae.application.healthcare.record.topology;

import org.resmedicinae.application.healthcare.record.*;
import org.resmedicinae.domain.healthcare.heading.problem.*;

import javax.swing.*;
import java.awt.*;
import java.util.*;

/**
 * This panel contains the current image, respectively shows it. The image has sensible regions where the user can
 * click on and move over. On click  the corresponding document will change.
 * On mouse move the specific overlayRegion will be highlighted.
 * @author Jens Bohl <info@jens-bohl-professional.de>
 */
public class ImagePanel extends RecordPanel {
    /** The image shown in the panel. */
    private Image image;
    /** The toolkit needed. Toolkits are used for loading images, such as JPG or GIF. */
    private Toolkit kit;
    /** The preferred size of the panel */
    private Dimension preferredSize;
    /** The maximum size of the panel */
    private Dimension maxSize;
    /** Vector of x-Coordinates of a overlayRegion dynamically filled when clicking on the image. */
    private Vector xCoordinates;
    /** Vector of y-Coordinates of a overlayRegion dynamically filled when clicking on the image. */
    private Vector yCoordinates;
    /** A vector containing all overlayRegions (refernces). Used for fast access on the regions.*/
    private Vector currentOverlayRegions;
    /** Vector of x-Coordinates of the marked region */
    private Vector markerXCoordinates;
    /** Vector of y-Coordinates of the marked region */
    private Vector markerYCoordinates;
    /** Reference to the current objective */
    private Objective currentObject;
    /**ToolTipManager for toolTips of regions. Not in use yet. Tooltips should appear in further versions as hints
     * over overlay regions.
     * */
    private ToolTipManager toolTipManager;

    /**Default constructor*/
    public ImagePanel() {
        setKit(Toolkit.getDefaultToolkit());
        setPreferredSize(new Dimension(390, 385));
        setMaximumSize(new Dimension(390, 285));
        setXCoordinates(new Vector());
        setYCoordinates(new Vector());
        setMarkerXCoordinates(new Vector());
        setMarkerYCoordinates(new Vector());
    }

    /**
     * Returns the image.
     * @return the image
     */
    public Image getImage() {
        return this.image;
    }

    /**
     * Sets the image.
     * @param image the image
     */
    public void setImage(Image image) {
        this.image = image;
    }

    /** Initializes this object. */
    public void initialize() {
        setBorder(BorderFactory.createLineBorder(Color.BLACK));
        addMouseListener((Record) getController());
        addMouseMotionListener((Record) getController());
        addKeyListener((Record) getController());
        addFocusListener((Record) getController());
        setToolTipManager(createToolTipManager());
    }

    /** Finalizes this object. */
    public void finalizz() {
        //??        super.finalizz();
    }

    /**
     * Paints the topology image.
     * @param g the graphics context
     * @exception NullPointerException if the graphics is null
     * @exception NullPointerException if the size is null
     */
    public void paintComponent(Graphics g) throws NullPointerException {
        super.paintComponent(g);
        g.setFont(new Font("LoadingFont",Font.BOLD,30));
        g.drawString("Loading Picture...", 50,190);
        RecordModel model = (RecordModel) getModel();
        if (g != null) {
            Dimension s = getSize();
            if ((s != null) && (getImage() != null)) {
                g.drawImage(getImage(), 0, 0, s.width, s.height, this);
                /*Draw also markers of the current object if there are any*/
                if (getCurrentObjective() != null) {
                    if (getCurrentObjective().hasMarkers()) {
                        drawMarkersForObject(getCurrentObjective());
                    }
                }
            }
        } else {
            throw new NullPointerException("Could not paint component. The graphics is null.");
        }
    }

    /**
     *Gets preferred size of JPanel
     *  @return the preferred size of the panel
     */
    public Dimension getPreferredSize() {
        return preferredSize;
    }

    /**
     *Gets maximum size of JPanel
     * @return the maximum size of the panel
     */
    public Dimension getMaximumSize() {
        return maxSize;
    }

    /**
     *Paints an overlayRegion with graphics context g. The color is red with a low alpha value, or solid or red.
     * This is some kind of highlighting the area used by registering mouse moves.
     * @param g the graphics context
     * @param overlayRegion the overlayRegion to be painted
     */
    public void paintOverlayRegion(Graphics g, OverlayRegion overlayRegion) {
        if (RecordModel.FILL_POLYGON_ALPHA) {
            g.setColor(new Color(.9f, 0f, 0f, 0.3f));
            g.fillPolygon(overlayRegion);
        } else if (RecordModel.FILL_POLYGON) {
            g.setColor(new Color(.9f, 0f, 0f));
            g.fillPolygon(overlayRegion);
        } else if (RecordModel.DRAW_POLYGON) {
            g.setColor(new Color(.9f, 0f, 0f));
            g.drawPolygon(overlayRegion);
        }
    }

    /** Paints the current overlayRegion represented by the vector xCoordinates and yCoordinates on the current overlay.*/
    public void paintCurrentRegionOnOverlay() {
        int[] x = new int[xCoordinates.size()];
        int[] y = new int[yCoordinates.size()];
        //vector to array
        for (int i = 0; i < xCoordinates.size(); i++) {
            x[i] = ((Integer) getXCoordinates().elementAt(i)).intValue();
            y[i] = ((Integer) getYCoordinates().elementAt(i)).intValue();
        }
        getGraphics().drawPolygon(x, y, xCoordinates.size());
    }

    /**
     *Clears the vectors xCoordinates and yCoordinates and repaints the panel. This only
     * removes a overlayRegion after it is drawn visually from the panel. This is no remove of an
     * overlayRegion from an document!!! Do not mix up that!!
     */
    public void deleteCurrentRegionFromOverlay() {
        xCoordinates.clear();
        yCoordinates.clear();
        update(getGraphics());
    }

    /**
     *Deletes the last drawn point from the panel. This means removing the last elements from vector xCoordinates and yCoordinates.
     */
    public void deleteLastPaintedPoint() {
        xCoordinates.remove(xCoordinates.lastElement());
        yCoordinates.remove(yCoordinates.lastElement());
        update(getGraphics());
        drawRegionPointsOnOverlay();
    }

    /** This method paints an overlayRegions. No solid drawing, only line borders. This is used for showing all
     * overlay regions.
     * @param overlayRegion the overlayRegion to be drawn
     *  */
    public void drawOverlayRegion(OverlayRegion overlayRegion) {
        getGraphics().drawPolygon(overlayRegion);
    }

    /** This method paints all points represented by values of the vectors xCoordinates and yCoordinates.
     * Not complete overlayRegions, only its points!!!
     * */
    public void drawRegionPointsOnOverlay() {
        for (int i = 0; i < xCoordinates.size(); i++) {
            int x = ((Integer) xCoordinates.elementAt(i)).intValue();
            int y = ((Integer) yCoordinates.elementAt(i)).intValue();
            getGraphics().drawRect(x, y, 3, 3);
        }
        if (xCoordinates.size() > 1) {
            for (int i = 0; i < xCoordinates.size() - 1; i++) {
                int x = ((Integer) xCoordinates.elementAt(i)).intValue();
                int y = ((Integer) yCoordinates.elementAt(i)).intValue();
                int next_x = ((Integer) xCoordinates.elementAt(i + 1)).intValue();
                int next_y = ((Integer) yCoordinates.elementAt(i + 1)).intValue();
                System.out.println("Redrawing line");
                getGraphics().drawLine(x, y, next_x, next_y);
            }
        }
    }

    /**
     *Gets the x coordinates that are currently in the panel.
     * @return vector containing the x coordinates of the current region
     */
    public Vector getOverlayRegionXCoordinates() {
        return this.xCoordinates;
    }

    /**
     *Gets the y coordinates that are currently in the panel.
     * @return vector containing the y coordinates of the current region
     */
    public Vector getOverlayRegionYCoordinates() {
        return this.yCoordinates;
    }

    /** Removes all markers from the overlay. Clearing vectors xCoordinates and yCoordinates.*/
    public void removeAllMarkersFromOverlay() {
        markerXCoordinates.clear();
        markerYCoordinates.clear();
    }

    /** Paints a marker at (x,y).
     * @param x the x position of the marker
     * @param y the y position of the marker
     * */
    public void drawMarker(int x, int y) {
        getGraphics().setColor(Color.RED);
        getGraphics().fillOval(x, y, 5, 5);
        markerXCoordinates.add(new Integer(x));
        markerYCoordinates.add(new Integer(y));
    }

    /**
     *Paints all markers for an object.
     * @param object the object for that the markers have to be painted
     */
    public void drawMarkersForObject(Objective object) {
        //removeAllMarkersFromOverlay();
        for (int i = 0; i < object.getMarkerCount(); i++) {
            getGraphics().fillOval(object.getXMarker(i).intValue(), object.getYMarker(i).intValue(), 5, 5);
        }
    }

    /** Updates the image. Forces the panel to redraw.*/
    public void updateImage() {
        update(getGraphics());
    }

    /**
     *Gets the tool kit
     * @return the tool kit
     */
    public Toolkit getKit() {
        return kit;
    }

    /**
     *Sets the tool kit
     * @param kit the tool kit
     */
    public void setKit(Toolkit kit) {
        this.kit = kit;
    }

    /**
     *Sets the preferred size of the image panel
     * @param preferredSize the preferred size
     */
    public void setPreferredSize(Dimension preferredSize) {
        this.preferredSize = preferredSize;
    }



    /**
     *Gets the x coordinates of the current region
     * @return vector containing the x coordinates
     */
    public Vector getXCoordinates() {
        return xCoordinates;
    }

    /**
     *Sets the x coordinates of the current region
     * @param xCoordinates vector containing the x coordinates
     */
    public void setXCoordinates(Vector xCoordinates) {
        this.xCoordinates = xCoordinates;
    }

    /**
     *Gets the y coordinates of the current region
     * @return vector containing the y coordinates
     */
    public Vector getYCoordinates() {
        return yCoordinates;
    }

    /**
     *Sets the y coordinates of the current region
     * @param yCoordinates vector containing the y coordinates
     */
    public void setYCoordinates(Vector yCoordinates) {
        this.yCoordinates = yCoordinates;
    }

    /**
     *Gets the current overlayRegions
     * @return vector containing the current overlayRegions on the overlay
     */
    public Vector getCurrentOverlayRegions() {
        return this.currentOverlayRegions;
    }

    /**
     *Sets the current overlayRegions
     * @param currentOverlayRegions a vector containing the current overlayRegions on the overlay
     */
    public void setCurrentOverlayRegions(Vector currentOverlayRegions) {
        this.currentOverlayRegions = currentOverlayRegions;
    }

    /**
     *Gets the x coordinates of the markers on this overlay
     * @return vector containing the x coordinates
     */
    public Vector getMarkerXCoordinates() {
        return markerXCoordinates;
    }

    /**
     *Sets the x coordinates of the markers on this overlay
     * @param xCoordinates vector containing the x coordinates
     */
    public void setMarkerXCoordinates(Vector markerXCoordinates) {
        this.markerXCoordinates = markerXCoordinates;
    }

    /**
     *Gets the y coordinates of the markers on this overlay
     * @return vector containing the y coordinates
     */
    public Vector getMarkerYCoordinates() {
        return markerYCoordinates;
    }

    /**
     *Sets the y coordinates of the markers on this overlay
     * @param yCoordinates vector containing the y coordinates
     */
    public void setMarkerYCoordinates(Vector markerYCoordinates) {
        this.markerYCoordinates = markerYCoordinates;
    }

    /**
     * Gets the current objective
     * @return the current objective
     */
    public Objective getCurrentObjective() {
        return currentObject;
    }

    /**
     * Sets the current objective
     * @param the current objective
     */
    public void setCurrentObjective(Objective currentObject) {
        this.currentObject = currentObject;
    }
    /**
     * Creates a toolTipManager
     * @return toolTipManager
     */
    public ToolTipManager createToolTipManager(){
        ToolTipManager t = ToolTipManager.sharedInstance();
        t.registerComponent(this);
        t.setEnabled(true);
        t.setInitialDelay(0);
        t.setLightWeightPopupEnabled(true);
        JToolTip tt = new JToolTip();
        tt.setComponent(this);
        tt.setTipText("TEXT");
        return t;
    }

    /**Gets the toolTipManager
     * @return the toolTipManager
     * */
    public ToolTipManager getToolTipManager() {
        return toolTipManager;
    }

    /**Sets the toolTipManager
     *@param toolTipManager the toolTipManager
     */
    public void setToolTipManager(ToolTipManager toolTipManager) {
        this.toolTipManager = toolTipManager;
    }
}
