/*
 * $RCSfile: DialogDisplayFactory.java,v $
 *
 * Copyright (c) 1999-2002. Christian Heller. All Rights Reserved.
 *
 * This software is published under the GPL GNU General Public License.
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * http://www.resmedicinae.org
 * - Information in Medicine -
 */

package org.resmedicinae.resmedlib.component.view.swing;

import java.awt.*;
import java.awt.event.*;
import java.lang.*;
import java.util.*;
import javax.swing.*;
import org.resmedicinae.resmedlib.component.view.*;

/**
 * This class represents a dialog display factory.
 *
 * @version $Revision: 1.6 $ $Date: 2002/06/12 18:49:39 $ $Author: chrissy $
 * @author Christian Heller <christian.heller@tuxtax.de>
 */
public class DialogDisplayFactory extends DisplayFactory {

    /** The parent window for the dialog. */
    private Window parentWindow;

    /**
     * Returns the parent window.
     *
     * @return the parent window
     */
    public Window getParentWindow() {

        return this.parentWindow;
    }

    /**
     * Sets the parent window.
     *
     * @param parentWindow the parent window
     */
    public void setParentWindow(Window parentWindow) {

        this.parentWindow = parentWindow;
    }

    /**
     * Creates dialog display.
     *
     * First, a new window is created.
     * Then, the window's root pane is initialized.
     *
     * The dialog will be parented to either:
     * - currently focussed window
     * - last opened window
     * - shared null frame
     *
     * @param v the swing view to create a display for
     * @param pv the parent application view
     * @return the root pane of the display
     * @exception NullPointerException if the dialog is null
     * @exception Exception if the determined parent window is neither a frame nor a dialog
     */
    public ResRootPane createDisplay(SwingView v, SwingView pv) throws NullPointerException, Exception {

        ResRootPane r = null;
        // Create a dialog to contain the view.
        ResDialog d = null;
        // Determine the parent window for the dialog.
        Window parentWindow = getParentWindow();

        if (parentWindow instanceof Frame) {

            d = new ResDialog((Frame) parentWindow);

        } else if (parentWindow instanceof Dialog) {

            d = new ResDialog((Dialog) parentWindow);

        } else {

            throw new Exception("Could not create display. The determined parent window is probably null. It is neither a frame nor a dialog.");
        }

        if (d != null) {

            r = (ResRootPane) d.getRootPane();

            initializeRootPane(r, v);
            initializeDisplay(d, v);

        } else {

            throw new NullPointerException("Could not create display. The dialog is null.");
        }

        return r;
    }

    /**
     * Destroys dialog display.
     *
     * @param v the swing view whose display is to be destroyed
     * @param pv the parent application view
     * @param r the root pane of the application view
     * @exception NullPointerException if the root pane is null
     */
    public void destroyDisplay(SwingView v, SwingView pv, ResRootPane r) throws NullPointerException {

        if (r != null) {

            ResDialog d = (ResDialog) r.getParent();

            finalizeRootPane(r, v);
            finalizeDisplay(d, v);

        } else {

            throw new NullPointerException("Could not destroy display. The root pane is null.");
        }
    }

    /**
     * Brings display to front.
     *
     * @param r the root pane of the application view whose display is to be brought to front
     * @exception NullPointerException if the root pane is null
     * @exception NullPointerException if the ... ?? is null
     */
    public void bringDisplayToFront(ResRootPane r) throws NullPointerException {
    }

    /**
     * Initializes the display.
     *
     * @param d the dialog
     * @param v the swing view
     * @exception NullPointerException if the dialog or the application view is null
     */
    private void initializeDisplay(ResDialog d, SwingView v) throws NullPointerException {

        if ((d != null) && (v != null)) {

            d.setTitle(v.getTitle());

            if (v.getDisplayMode() == DisplayFactory.MODAL_DIALOG_DISPLAY) {

                d.setModal(true);

            } else {

                d.setModal(false);
            }

            d.setBounds(v.getDialogBounds());
            d.setResizable(v.getResizable());
            d.addWindowListener(new WindowDisplayAdapter(v));

            // Don't let Swing handle the window close. Nasty... Swing needs an interface here.
/*??
            if (w instanceof ResFrame) {

                ((ResFrame) w).setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);

            } else if (w instanceof ResDialog) {

                ((ResDialog) w).setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
            }
*/

/*??
            // Display safely from event queue.
            org.scopemvc.view.swing.SwingUtil.runFromSwingEventThread(

                new Runnable() {

                    public void run() {

                        d.setVisible(true);
                    }
                }
            );
*/

            d.show();

        } else {

            throw new NullPointerException("Could not initialize display. The dialog or the swing view is null.");
        }
    }

    /**
     * Finalizes the display.
     *
     * @param d the dialog
     * @param v the swing view
     * @exception NullPointerException if the dialog or the application view is null
     */
    private void finalizeDisplay(ResDialog d, SwingView v) throws NullPointerException {

        if ((d != null) && (v != null)) {

            d.hide();

            WindowListener[] wl = d.getWindowListeners();

            if (wl != null) {

                for (int i = 0; i < wl.length; i++) {

                    d.removeWindowListener(wl[i]);
                }

            } else {

                throw new NullPointerException("Could not finalize window listeners. The window listener array is null.");
            }

            v.setResizable(d.isResizable());
            v.setDialogBounds(d.getBounds());

            if (d.isModal()) {

                v.setDisplayMode(DisplayFactory.MODAL_DIALOG_DISPLAY);

            } else {

                v.setDisplayMode(DisplayFactory.MODELESS_DIALOG_DISPLAY);
            }

            v.setTitle(d.getTitle());

/*??
            org.scopemvc.view.swing.SwingUtil.runFromSwingEventThread(new Runnable() {

                public void run() {

                    // Release all of the native screen resources used by this Window,
                    // its subcomponents, and all of its owned children.
                    w.dispose();
                }
            });
*/
        } else {

            throw new NullPointerException("Could not finalize display. The dialog or the swing view is null.");
        }
    }
}

